"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
var formatTokens_1 = __importDefault(require("./formatTokens"));
var Formater = /** @class */ (function () {
    function Formater(duration, input, options) {
        if (options === void 0) { options = {}; }
        var _this = this;
        this.duration = duration;
        this.input = input;
        this.options = options;
        this._hour = 0;
        this._minute = 0;
        this._second = 0;
        this._milliSecond = 0;
        this._formatTokens = {};
        this.calcHour = function () {
            _this._hour = Math.floor(_this.duration.millisecond / 3600000);
            return _this._hour;
        };
        this.calcMin = function () {
            _this._minute = Math.floor((_this.duration.millisecond - _this._hour * 3600000) / 60000);
            return _this._minute;
        };
        this.calcSec = function () {
            _this._second = Math.floor((_this.duration.millisecond - _this._hour * 3600000
                - _this._minute * 60000) / 1000);
            return _this._second;
        };
        this.calcMilliSec = function () {
            _this._milliSecond = _this.duration.millisecond - _this._hour * 3600000
                - _this._minute * 60000 - _this._second * 1000;
            return _this._milliSecond;
        };
        var inputTokens = input.match(Formater.FORMAT_EXPRESSION);
        if (inputTokens === null)
            throw new Error('invalid token!');
        this._inputTokens = inputTokens.map(function (token) {
            var type = formatTokens_1.default.formatTokens.filter(function (types) { return types.token === token; });
            return type.length === 0 ? { type: 'text', token: token, value: '' } : type[0];
        });
        this.addFormatToken('h', 'hour', this.calcHour);
        this.addFormatToken('hh', 'hour', this.calcHour, 2);
        this.addFormatToken('m', 'minute', this.calcMin);
        this.addFormatToken('mm', 'minute', this.calcMin, 2);
        this.addFormatToken('s', 'second', this.calcSec);
        this.addFormatToken('ss', 'second', this.calcSec, 2);
        this.addFormatToken('S', 'millisecond', this.calcMilliSec);
    }
    Formater.prototype.addFormatToken = function (token, type, func, pad) {
        if (pad === void 0) { pad = 0; }
        this._formatTokens[token] = {
            type: type,
            func: func,
            pad: pad,
        };
    };
    Formater.prototype.formatValue = function (token) {
        var value = this._formatTokens[token].func();
        if (this.options.digitSeparator) {
            value = this.formatNumber(value);
        }
        value = Formater.zeroPad(String(value), this._formatTokens[token].pad);
        return value;
    };
    Formater.zeroPad = function (value, length) {
        var s = value;
        if (s.length >= length)
            return s;
        while (s.length < length)
            s = "0" + s;
        return s;
    };
    Formater.prototype.formatNumber = function (value) {
        var parts = String(value).split('.');
        parts[0] = parts[0].replace(/(\d)(?=(\d{3})+(?!\d))/g, "$1" + this.options.digitSeparator);
        return parts.join('.');
    };
    Formater.prototype.formatFunction = function (token) {
        if (this._formatTokens[token] && typeof this._formatTokens[token].func === 'function') {
            return this.formatValue(token);
        }
        return token.replace(/^\[/, '').replace(/\]$/, '');
    };
    Formater.prototype.format = function () {
        var _this = this;
        if (this._inputTokens === null)
            return '';
        Formater.TYPE_ORDER.forEach(function (type) {
            _this._inputTokens.forEach(function (inputToken) {
                if (type === inputToken.type) {
                    // eslint-disable-next-line no-param-reassign
                    inputToken.value = _this.formatFunction(inputToken.token);
                }
            });
        });
        return this._inputTokens.map(function (inputToken) { return inputToken.value; }).join('');
    };
    Formater.FORMAT_EXPRESSION = /\[.+?\]|\*?[Hh]+|\*?m+|\*?s+|\*?S|./g;
    Formater.TYPE_ORDER = ['hour', 'minute', 'second', 'millisecond', 'text'];
    return Formater;
}());
exports.default = Formater;
